@@ -94,6 +94,8 @@ ExoBindingLink;
  *
  * Opaque structure representing a one-way binding between two properties.
  * It is automatically removed if one of the bound objects is finalized.
+ *
+ * Deprecated: xfce 4.18: In favor of GBinding
  **/
 struct _ExoBinding
 {
@@ -107,6 +109,8 @@ struct _ExoBinding
  *
  * Opaque structure representing a mutual binding between two properties.
  * It is automatically freed if one of the bound objects is finalized.
+ *
+ * Deprecated: xfce 4.18: In favor of GBinding
  **/
 struct _ExoMutualBinding
 {
@@ -302,6 +306,8 @@ exo_binding_link_init (ExoBindingLink     *blink,
  * Before binding the value of @dst_property is set to the
  * value of @src_property.
  *
+ * Deprecated: xfce 4.18: In favor of GBinding
+ *
  * Returns: The descriptor of the binding. It is automatically
  *          removed if one of the objects is finalized.
  **/
@@ -334,6 +340,8 @@ exo_binding_new (GObject      *src_object,
  * Before binding the value of @dst_property is set to the
  * value of @src_property.
  *
+ * Deprecated: xfce 4.18: In favor of GBinding
+ *
  * Returns: The descriptor of the binding. It is automatically
  *          removed if one of the objects is finalized.
  **/
@@ -395,6 +403,8 @@ exo_binding_new_full (GObject            *src_object,
  *
  * Convenience function for binding with boolean negation of value.
  *
+ * Deprecated: xfce 4.18: In favor of GBinding
+ *
  * Returns: The descriptor of the binding. It is automatically
  *          removed if one of the objects is finalized.
  **/
@@ -404,11 +414,13 @@ exo_binding_new_with_negation (GObject      *src_object,
                                GObject      *dst_object,
                                const gchar  *dst_property)
 {
+  G_GNUC_BEGIN_IGNORE_DEPRECATIONS
   ExoBindingTransform transform = (ExoBindingTransform) (void (*)(void)) exo_g_value_transform_negate;
 
   return exo_binding_new_full (src_object, src_property,
                                dst_object, dst_property,
                                transform, NULL, NULL);
+  G_GNUC_END_IGNORE_DEPRECATIONS
 }
 
 
@@ -422,6 +434,8 @@ exo_binding_new_with_negation (GObject      *src_object,
  *
  * This functions also calls the @destroy_notify function that
  * was specified when @binding was created.
+ *
+ * Deprecated: xfce 4.18: In favor of GBinding
  **/
 void
 exo_binding_unbind (ExoBinding *binding)
@@ -443,6 +457,8 @@ exo_binding_unbind (ExoBinding *binding)
  * Before binding the value of @property2 is set to the value
  * of @property1.
  *
+ * Deprecated: xfce 4.18: In favor of GBinding
+ *
  * Returns: The descriptor of the binding. It is automatically
  *          removed if one of the objects is finalized.
  **/
@@ -479,6 +495,8 @@ exo_mutual_binding_new (GObject     *object1,
  * %NULL or non-%NULL. If they are non-%NULL, they should be reverse
  * in each other.
  *
+ * Deprecated: xfce 4.18: In favor of GBinding
+ *
  * Returns: The descriptor of the binding. It is automatically
  *          removed if one of the objects is finalized.
  **/
@@ -558,6 +576,8 @@ exo_mutual_binding_new_full (GObject            *object1,
  *
  * Convenience function for binding with boolean negation of value.
  *
+ * Deprecated: xfce 4.18: In favor of GBinding
+ *
  * Returns: The descriptor of the binding. It is automatically removed
  *          if one of the objects if finalized.
  **/
@@ -567,12 +587,14 @@ exo_mutual_binding_new_with_negation (GObject     *object1,
                                       GObject     *object2,
                                       const gchar *property2)
 {
+  G_GNUC_BEGIN_IGNORE_DEPRECATIONS
   ExoBindingTransform transform = (ExoBindingTransform) (void (*)(void)) exo_g_value_transform_negate;
 
   return exo_mutual_binding_new_full (object1, property1,
                                       object2, property2,
                                       transform, transform,
                                       NULL, NULL);
+  G_GNUC_END_IGNORE_DEPRECATIONS
 }
 
 
@@ -586,6 +608,8 @@ exo_mutual_binding_new_with_negation (GObject     *object1,
  *
  * This functions also calls the @destroy_notify function that
  * was specified when @binding was created.
+ *
+ * Deprecated: xfce 4.18: In favor of GBinding
  **/
 void
 exo_mutual_binding_unbind (ExoMutualBinding *binding)
