@@ -51,9 +51,7 @@
  **/
 
 /* HACK: fix dead API via #define */
-#if GTK_CHECK_VERSION (3, 0, 0)
 # define gtk_icon_info_free(info) g_object_unref (info)
-#endif
 
 /* Property identifiers */
 enum
@@ -78,31 +76,17 @@ static void exo_cell_renderer_icon_set_property (GObject                  *objec
                                                  GParamSpec               *pspec);
 static void exo_cell_renderer_icon_get_size     (GtkCellRenderer          *renderer,
                                                  GtkWidget                *widget,
-#if GTK_CHECK_VERSION (3, 0, 0)
                                                  const GdkRectangle       *cell_area,
-#else
-                                                 GdkRectangle             *cell_area,
-#endif
                                                  gint                     *x_offset,
                                                  gint                     *y_offset,
                                                  gint                     *width,
                                                  gint                     *height);
-#if GTK_CHECK_VERSION (3, 0, 0)
 static void exo_cell_renderer_icon_render       (GtkCellRenderer          *renderer,
                                                  cairo_t                  *cr,
                                                  GtkWidget                *widget,
                                                  const GdkRectangle       *background_area,
                                                  const GdkRectangle       *cell_area,
                                                  GtkCellRendererState      flags);
-#else
-static void exo_cell_renderer_icon_render       (GtkCellRenderer          *renderer,
-                                                 GdkWindow                *window,
-                                                 GtkWidget                *widget,
-                                                 GdkRectangle             *background_area,
-                                                 GdkRectangle             *cell_area,
-                                                 GdkRectangle             *expose_area,
-                                                 GtkCellRendererState      flags);
-#endif
 
 
 
@@ -328,11 +312,7 @@ exo_cell_renderer_icon_set_property (GObject      *object,
 static void
 exo_cell_renderer_icon_get_size (GtkCellRenderer *renderer,
                                  GtkWidget       *widget,
-#if GTK_CHECK_VERSION (3, 0, 0)
                                  const GdkRectangle *cell_area,
-#else
-                                 GdkRectangle    *cell_area,
-#endif
                                  gint            *x_offset,
                                  gint            *y_offset,
                                  gint            *width,
@@ -377,7 +357,6 @@ exo_cell_renderer_icon_get_size (GtkCellRenderer *renderer,
 }
 
 
-#if GTK_CHECK_VERSION (3, 0, 0)
 static void
 exo_cell_renderer_icon_render (GtkCellRenderer     *renderer,
                                cairo_t             *cr,
@@ -386,25 +365,11 @@ exo_cell_renderer_icon_render (GtkCellRenderer     *renderer,
                                const GdkRectangle  *cell_area,
                                GtkCellRendererState flags)
 {
-    GdkRectangle        clip_area;
-    GdkRectangle       *expose_area = &clip_area;
-    GdkRGBA            *color_rgba;
-    GdkColor            color_gdk;
-    GtkStyleContext    *style_context;
-#else
-static void
-exo_cell_renderer_icon_render (GtkCellRenderer     *renderer,
-                               GdkWindow           *window,
-                               GtkWidget           *widget,
-                               GdkRectangle        *background_area,
-                               GdkRectangle        *cell_area,
-                               GdkRectangle        *expose_area,
-                               GtkCellRendererState flags)
-{
-  cairo_t *cr = gdk_cairo_create (window);
-  GtkIconSource                    *icon_source;
-  GtkStateType                      state;
-#endif
+  GdkRectangle        clip_area;
+  GdkRectangle       *expose_area = &clip_area;
+  GdkRGBA            *color_rgba;
+  GdkColor            color_gdk;
+  GtkStyleContext    *style_context;
   const ExoCellRendererIconPrivate *priv = exo_cell_renderer_icon_get_instance_private (EXO_CELL_RENDERER_ICON (renderer));
   GtkIconTheme                     *icon_theme;
   GdkRectangle                      icon_area;
@@ -413,25 +378,29 @@ exo_cell_renderer_icon_render (GtkCellRenderer     *renderer,
   GtkIconInfo                      *icon_info = NULL;
   GdkPixbuf                        *icon = NULL;
   GdkPixbuf                        *temp;
+  cairo_surface_t                  *surface;
   GError                           *err = NULL;
   gchar                            *display_name = NULL;
-  gint                             *icon_sizes;
-  gint                              icon_size;
+  gint                             *theme_icon_sizes;
+  gint                              requested_icon_size;
+  gint                              selected_icon_size;
+  gint                              scale_factor;
   gint                              n;
 
-#if GTK_CHECK_VERSION (3, 0, 0)
   gdk_cairo_get_clip_rectangle (cr, expose_area);
-#endif
 
   /* verify that we have an icon */
   if (G_UNLIKELY (priv->icon == NULL && priv->gicon == NULL))
     return;
 
+  scale_factor = gtk_widget_get_scale_factor (widget);
+  requested_icon_size = priv->size * scale_factor;
+
   /* icon may be either an image file or a named icon */
   if (priv->icon != NULL && g_path_is_absolute (priv->icon))
     {
       /* load the icon via the thumbnail database */
-      icon = _exo_thumbnail_get_for_file (priv->icon, (priv->size > 128) ? EXO_THUMBNAIL_SIZE_LARGE : EXO_THUMBNAIL_SIZE_NORMAL, &err);
+      icon = _exo_thumbnail_get_for_file (priv->icon, (requested_icon_size > 128) ? EXO_THUMBNAIL_SIZE_LARGE : EXO_THUMBNAIL_SIZE_NORMAL, &err);
     }
   else if (priv->icon != NULL || priv->gicon != NULL)
     {
@@ -440,30 +409,37 @@ exo_cell_renderer_icon_render (GtkCellRenderer     *renderer,
 
       if (priv->icon != NULL)
         {
-          icon_sizes = gtk_icon_theme_get_icon_sizes (icon_theme, priv->icon);
-          for (icon_size = -1, n = 0; icon_sizes[n] != 0; ++n)
+          theme_icon_sizes = gtk_icon_theme_get_icon_sizes (icon_theme, priv->icon);
+          for (selected_icon_size = -1, n = 0; theme_icon_sizes[n] != 0; ++n)
             {
-              /* we can use any size if scalable, because we load the file directly */
-              if (icon_sizes[n] == -1)
-                icon_size = priv->size;
-              else if (icon_sizes[n] > icon_size && icon_sizes[n] <= priv->size)
-                icon_size = icon_sizes[n];
+              /* we can use any size if scalable, because we load the file directly. also bail if we get an exact match. */
+              if (theme_icon_sizes[n] == -1 || theme_icon_sizes[n] == requested_icon_size)
+                {
+                  selected_icon_size = requested_icon_size;
+                  break;
+                }
+              /* selected icon size is still too small, but this icon size is better */
+              else if (selected_icon_size < requested_icon_size && theme_icon_sizes[n] > selected_icon_size)
+                selected_icon_size = theme_icon_sizes[n];
+              /* this icon is large enough, and is a closer match than what we've selected */
+              else if (theme_icon_sizes[n] > requested_icon_size && theme_icon_sizes[n] < selected_icon_size)
+                selected_icon_size = theme_icon_sizes[n];
             }
-          g_free (icon_sizes);
+          g_free (theme_icon_sizes);
 
           /* if we don't know any icon sizes at all, the icon is probably not present */
-          if (G_UNLIKELY (icon_size < 0))
-            icon_size = priv->size;
+          if (G_UNLIKELY (selected_icon_size < 0))
+            selected_icon_size = requested_icon_size;
 
           /* lookup the icon in the icon theme */
-          icon_info = gtk_icon_theme_lookup_icon (icon_theme, priv->icon, icon_size, 0);
+          icon_info = gtk_icon_theme_lookup_icon (icon_theme, priv->icon, selected_icon_size, GTK_ICON_LOOKUP_FORCE_SIZE);
         }
       else if (priv->gicon != NULL)
         {
           icon_info = gtk_icon_theme_lookup_by_gicon (icon_theme,
                                                       priv->gicon,
-                                                      priv->size,
-                                                      GTK_ICON_LOOKUP_USE_BUILTIN);
+                                                      requested_icon_size,
+                                                      GTK_ICON_LOOKUP_USE_BUILTIN | GTK_ICON_LOOKUP_FORCE_SIZE);
         }
 
       if (G_UNLIKELY (icon_info == NULL))
@@ -477,7 +453,7 @@ exo_cell_renderer_icon_render (GtkCellRenderer     *renderer,
            * real available cell area directly here, because loading thumbnails involves scaling anyway
            * and this way we need to the thumbnail pixbuf scale only once.
            */
-          icon = _exo_thumbnail_get_for_file (filename, (priv->size > 128) ? EXO_THUMBNAIL_SIZE_LARGE : EXO_THUMBNAIL_SIZE_NORMAL, &err);
+          icon = _exo_thumbnail_get_for_file (filename, (requested_icon_size > 128) ? EXO_THUMBNAIL_SIZE_LARGE : EXO_THUMBNAIL_SIZE_NORMAL, &err);
         }
       else
         {
@@ -513,26 +489,25 @@ exo_cell_renderer_icon_render (GtkCellRenderer     *renderer,
     }
 
   /* determine the real icon size */
-  icon_area.width = gdk_pixbuf_get_width (icon);
-  icon_area.height = gdk_pixbuf_get_height (icon);
+  icon_area.width = gdk_pixbuf_get_width (icon) / scale_factor;
+  icon_area.height = gdk_pixbuf_get_height (icon) / scale_factor;
 
   /* scale down the icon on-demand */
   if (G_UNLIKELY (icon_area.width > cell_area->width || icon_area.height > cell_area->height))
     {
       /* scale down to fit */
-      temp = exo_gdk_pixbuf_scale_down (icon, TRUE, cell_area->width, cell_area->height);
+      temp = exo_gdk_pixbuf_scale_down (icon, TRUE, cell_area->width * scale_factor, cell_area->height * scale_factor);
       g_object_unref (G_OBJECT (icon));
       icon = temp;
 
       /* determine the icon dimensions again */
-      icon_area.width = gdk_pixbuf_get_width (icon);
-      icon_area.height = gdk_pixbuf_get_height (icon);
+      icon_area.width = gdk_pixbuf_get_width (icon) / scale_factor;
+      icon_area.height = gdk_pixbuf_get_height (icon) / scale_factor;
     }
 
   icon_area.x = cell_area->x + (cell_area->width - icon_area.width) / 2;
   icon_area.y = cell_area->y + (cell_area->height - icon_area.height) / 2;
 
-  /* Gtk2: check whether the icon is affected by the expose event */
   /* Gtk3: we don't have any expose rectangle and just draw everything */
   if (gdk_rectangle_intersect (expose_area, &icon_area, &draw_area))
     {
@@ -541,7 +516,6 @@ exo_cell_renderer_icon_render (GtkCellRenderer     *renderer,
         {
           if ((flags & GTK_CELL_RENDERER_SELECTED) != 0)
             {
-#if GTK_CHECK_VERSION (3, 0, 0)
               style_context = gtk_widget_get_style_context (widget);
               gtk_style_context_get (style_context, gtk_widget_has_focus (widget) ? GTK_STATE_FLAG_SELECTED : GTK_STATE_FLAG_ACTIVE,
                                      GTK_STYLE_PROPERTY_BACKGROUND_COLOR, &color_rgba,
@@ -553,11 +527,6 @@ exo_cell_renderer_icon_render (GtkCellRenderer     *renderer,
               color_gdk.green = color_rgba->green * 65535;
               gdk_rgba_free (color_rgba);
               temp = exo_gdk_pixbuf_colorize (icon, &color_gdk);
-#else
-              state = gtk_widget_has_focus (widget) ? GTK_STATE_SELECTED : GTK_STATE_ACTIVE;
-
-              temp = exo_gdk_pixbuf_colorize (icon, &gtk_widget_get_style (widget)->base[state]);
-#endif
               g_object_unref (G_OBJECT (icon));
               icon = temp;
             }
@@ -570,7 +539,6 @@ exo_cell_renderer_icon_render (GtkCellRenderer     *renderer,
             }
         }
 
-#if GTK_CHECK_VERSION (3, 0, 0)
       /* check if we should render an insensitive icon */
       if (G_UNLIKELY (gtk_widget_get_state_flags(widget) & GTK_STATE_INSENSITIVE || !gtk_cell_renderer_get_sensitive (renderer)))
         {
@@ -589,39 +557,18 @@ exo_cell_renderer_icon_render (GtkCellRenderer     *renderer,
           g_object_unref (G_OBJECT (icon));
           icon = temp;
         }
-#else
-      /* check if we should render an insensitive icon */
-      if (G_UNLIKELY (gtk_widget_get_state (widget) == GTK_STATE_INSENSITIVE || !gtk_cell_renderer_get_sensitive (renderer)))
-        {
-          /* allocate an icon source */
-          icon_source = gtk_icon_source_new ();
-          gtk_icon_source_set_pixbuf (icon_source, icon);
-          gtk_icon_source_set_size_wildcarded (icon_source, FALSE);
-          gtk_icon_source_set_size (icon_source, GTK_ICON_SIZE_SMALL_TOOLBAR);
-
-          /* render the insensitive icon */
-          temp = gtk_style_render_icon (gtk_widget_get_style (widget), icon_source, gtk_widget_get_direction (widget),
-                                        GTK_STATE_INSENSITIVE, -1, widget, "gtkcellrendererpixbuf");
-          g_object_unref (G_OBJECT (icon));
-          icon = temp;
-
-          /* release the icon source */
-          gtk_icon_source_free (icon_source);
-        }
-#endif
 
       /* render the invalid parts of the icon */
-      gdk_cairo_set_source_pixbuf (cr, icon, icon_area.x, icon_area.y);
+      surface = gdk_cairo_surface_create_from_pixbuf (icon, scale_factor, gtk_widget_get_window (widget));
+      cairo_set_source_surface (cr, surface, icon_area.x, icon_area.y);
       cairo_rectangle (cr, draw_area.x, draw_area.y, draw_area.width, draw_area.height);
       cairo_fill (cr);
+
+      cairo_surface_destroy (surface);
     }
 
   /* release the file's icon */
   g_object_unref (G_OBJECT (icon));
-
-#if !GTK_CHECK_VERSION (3, 0, 0)
-  cairo_destroy (cr);
-#endif
 }
 
 
