@@ -1,4 +1,3 @@
-/* $Id$ */
 /*-
  * Copyright (c) 2004 os-cillation e.K.
  *
@@ -14,10 +13,10 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Library General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+ * MA 02110-1301 USA
  */
 
 #if !defined (EXO_INSIDE_EXO_H) && !defined (EXO_COMPILATION)
@@ -31,6 +30,7 @@
 
 G_BEGIN_DECLS
 
+G_DEPRECATED
 gboolean  exo_g_value_transform_negate  (const GValue *src_value,
                                          GValue       *dst_value);
 
