@@ -30,8 +30,8 @@
 #include <exo/exo-config.h>
 #include <exo/exo-job.h>
 #include <exo/exo-private.h>
-#include <exo/exo-string.h>
 #include <exo/exo-alias.h>
+#include <libxfce4util/libxfce4util.h>
 
 /**
  * SECTION: exo-job
